package org.openmuc.openiec61850.sample;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.openmuc.openiec61850.ClientAssociation;
import org.openmuc.openiec61850.ClientSap;
import org.openmuc.openiec61850.ServerModel;
import org.openmuc.openiec61850.ServiceError;
import org.openmuc.openiec61850.Urcb;
import org.openmuc.openiec61850.client.Report;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author Stefan Feuerhahn
 * 
 *         Note that this class is for demonstration purposes only. Since the OpenIEC61850 server does not support
 *         reporting yet it cannot be run in combination with the SampleServer.
 * 
 */
public class SampleReportClient extends Thread {

	private final static Logger logger = LoggerFactory.getLogger(SampleReportClient.class);

	private static ClientAssociation association = null;

	public static void main(String[] args) throws InterruptedException, ServiceError, IOException {

		String usageString = "usage: org.openmuc.openiec61850.sample.SampleReportClient <host> <port>";

		if (args.length != 2) {
			System.out.println(usageString);
			return;
		}

		String remoteHost = args[0];
		InetAddress address;
		try {
			address = InetAddress.getByName(remoteHost);
		} catch (UnknownHostException e) {
			logger.error("Unknown host: " + remoteHost);
			return;
		}

		int remotePort;
		try {
			remotePort = Integer.parseInt(args[1]);
		} catch (NumberFormatException e) {
			System.out.println(usageString);
			return;
		}

		ClientSap clientSap = new ClientSap();
		// alternatively you could use ClientSap(SocketFactory factory) to e.g. connect using SSL

		// optionally you can set some association parameters (but usually the default should work):
		// clientSap.setTSelRemote(new byte[] { 0, 1 });
		// clientSap.setTSelLocal(new byte[] { 0, 0 });

		logger.info("Attempting to connect to server " + remoteHost + " on port " + remotePort);
		try {
			association = clientSap.associate(address, remotePort, null);
		} catch (IOException e) {
			// an IOException will always indicate a fatal exception. It indicates that the association was closed and
			// cannot be recovered. You will need to create a new association using ClientSap.associate() in order to
			// reconnect.
			logger.error("Error connecting to server: " + e.getMessage());
			return;
		}

		ServerModel serverModel;
		try {
			// requestModel() will call all GetDirectory and GetDefinition ACSI services needed to get the complete
			// server model
			serverModel = association.retrieveModel();
		} catch (ServiceError e) {
			logger.error("Service Error requesting model.", e);
			association.close();
			return;
		} catch (IOException e) {
			logger.error("Fatal IOException requesting model.", e);
			return;
		}

		// instead of calling retrieveModel you could read the model directly from an SCL file:
		// try {
		// serverModel = association.getModelFromSclFile("../sampleServer/sampleModel.icd");
		// } catch (SclParseException e1) {
		// logger.error("Error parsing SCL file.", e1);
		// return;
		// }

		SampleReportClient reportHandlerThread = new SampleReportClient();
		reportHandlerThread.start();

		association.enableReportQueuing();

		Urcb urcb = serverModel.getUrcb("E1Test/LLN0.urcbST");

		if (urcb == null) {
			logger.error("ReportControlBlock not found");
		}
		else {
			association.getRcbValues(urcb);
			logger.debug("urcb name: " + urcb.getName());
			logger.debug("RptId: " + urcb.getRptId());
			logger.debug("RptEna: " + urcb.getRptEna().getValue());

			association.enableReporting(urcb);
			association.startGi(urcb);

		}

	}

	@Override
	public void run() {
		Report report = null;

		try {
			report = association.getReport();
		} catch (ServiceError e) {
			e.printStackTrace();
			return;
		}

		logger.debug("got report with dataset ref: " + report.getDataSet().getReferenceStr());
		// do something with the report

	}

}
